/////////////////////////////////////////////////////////////////
// Name: xGraphics.h
/////////////////////////////////////////////////////////////////
// Description:
// - Core graphics system that handles rendering of all
//   xRenderable objects in the current xSceneGraph object
/////////////////////////////////////////////////////////////////
// - 1.8.2004, Update, Arto Ruotsalainen (Arex)
//   Added iFrame to CreateTexture, for single textures
//   just set it to zero, it is for multiple frame objects. 
//
// - 15.7.2004, Update, Arto Ruotsalainen (Arex)
//   Added get device pointer function
//
// - 13.7.2004, Update, Arto Ruotsalainen (Arex)
//   Added screen width and height variables.
//
// - 7.10.2004, Update, Dayle Flowers(Xienen)
//   Added xPool for Objects and Create2DObject() function
//   Added AlphaBlit and Goto2DMode functions
//
// - 7.6.2004, Update, Arto Ruotsalainen(Arex)
//   Comment stuff added to begin of the file.
//   Add new updates/fixes above this, so the old updates
//   will be at the bottom.
/////////////////////////////////////////////////////////////////
#ifndef XGRAPHICS_H_
#define XGRAPHICS_H_

#include <stdlib.h>
#include <AGL/agl.h>
#include <OpenGL/glu.h>
#include <OpenGL/glext.h>

#include "xResults.h"
#include "xTexture.h"
#include "xVector.h"

#include "xRenderData.h"
#include "xSceneGraph.h"

typedef UInt32 DWORD;

//#pragma comment(lib, "opengl32.lib")
//#pragma comment(lib, "glu32.lib")
//#pragma comment(lib, "glaux.lib")

struct xGraphics
{
	WindowPtr	m_pWindow;

	int			m_iScreenWidth;
	int			m_iScreenHeight;
	DWORD		m_dwDisplayFrequency;
	float		m_fGrayScaleFactor;
	CFArrayRef	m_arrayDisplayModes;
	int			m_iDisplayModeCount;
	int			m_iAllScreenResolutionsIndex;

	AGLDrawable	 m_hDrawable;
	AGLContext	 m_hContext;
	xSceneGraph	*m_pSceneGraph;
	xTexture	 m_xTexturePool;

	int m_iFrameSkip;

	//Shader Indices
	GLuint m_uiAlphaKillPixelShaderIndex;
	GLuint m_uiAlphaMaskPixelShaderIndex;
	GLuint m_uiGrayScalePixelShaderIndex;
	GLuint m_uiDualColorPixelShaderIndex;

#ifdef __FRAGSHADER_NOARB
	GLuint m_uiAlphaKillPixelProgramIndex;
	GLuint m_uiAlphaMaskPixelProgramIndex;
	GLuint m_uiGrayScalePixelProgramIndex;
	GLuint m_uiDualColorPixelProgramIndex;

// Alpha Kill Pixel shader
	struct
	{
		GLint m_iTexture;
	} m_xAlphaKillPixel;

// Alpha Mask Pixel shader
	struct
	{
		GLint m_iTextureRatio;
		GLint m_iTextureOffset;
		GLint m_iGrayScaleFactor;
		GLint m_iMask;
		GLint m_iTexture;
	} m_xAlphaMaskPixel;

// GrayScale Pixel shader
	struct
	{
		GLint m_iGrayScaleFactor;
		GLint m_iTexture;
	} m_xGrayScalePixel;

// Dual Color Pixel shader
	struct
	{
		GLint m_iSecondColor;
		GLint m_iColorDifference;
		GLint m_iGrayScaleFactor;
		GLint m_iTexture;
	} m_xDualColorPixel;
#endif // __FRAGSHADER_NOARB
};

extern xGraphics g_xGraphics;

FORCE_INLINE void xGraphicsSetSceneGraph(xSceneGraph *pSceneGraph) { g_xGraphics.m_pSceneGraph = pSceneGraph; }

int xGraphicsInit(WindowPtr pWindow, int iWindowWidth, int iWindowHeight, bool bFullscreenApp, int iBPP);
void xGraphicsPreUpdate(DWORD dwTime);
void xGraphicsUpdate(double dTime);
void xGraphicsShutdown();

void xGraphicsGetDesktopResolution(int *pScreenWidth, int *pScreenHeight);
void xGraphicsStartGettingAllDesktopResolutions();
bool xGraphicsGetNextDesktopResolution(int *pScreenWidth, int *pScreenHeight);
bool xGraphicsIsResolutionAvailable(int iScreenWidth, int iScreenHeight, bool bFullscreen);

xTexture* xGraphicsGetTexture(const char* strFilePath, int iFrame);
int xGraphicsGetTextures(const char* strFilePath, xTexture ***pTextures, float *fAnimationRate = 0);
void xGraphicsReloadTexturePool();
void xGraphicsDeleteTexture(xTexture* pTexture);
void xGraphicsDeleteTextures(xTexture **pTextures, int iCount);

FORCE_INLINE void xGraphicsDeleteTextureReference(xTexture* pTexture)
{
	if(pTexture->m_iReferenceCount > 0)
		pTexture->m_iReferenceCount--;
}

FORCE_INLINE xTexture* xGraphicsTexturePoolNew(const char* strFilePath)
{
	xTexture *current = &g_xGraphics.m_xTexturePool;

	while(current->m_pNext)
	{
		if(!stricmp(current->m_strObjectPath, strFilePath))
		{
			current->m_iReferenceCount++;
			return current;
		}

		current = current->m_pNext;
	}

	if(!stricmp(current->m_strObjectPath, strFilePath))
	{
		current->m_iReferenceCount++;
		return current;
	}

	current->m_pNext = (xTexture*)dlmalloc(sizeof(xTexture));
	xTextureInit(current->m_pNext);
	current->m_pNext->m_pPrev = current;
	
	strcpy(current->m_pNext->m_strObjectPath, strFilePath);
	return current->m_pNext;
}

FORCE_INLINE xTexture* xGraphicsTexturePoolGet(const char* strFilePath)
{
	xTexture *current = &g_xGraphics.m_xTexturePool;

	while(current->m_pNext)
	{
		if(!stricmp(current->m_strObjectPath, strFilePath))
		{
			current->m_iReferenceCount++;
			return current;
		}

		current = current->m_pNext;
	}

	if(!stricmp(current->m_strObjectPath, strFilePath))
	{
		current->m_iReferenceCount++;
		return current;
	}

	return 0;
}

FORCE_INLINE void xGraphicsGoto2DMode()
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	glOrtho(0, g_xGraphics.m_iScreenWidth, g_xGraphics.m_iScreenHeight, 0, -1, 1);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glDisable(GL_DEPTH_TEST);
	//glDepthMask(GL_FALSE);

	glDisable(GL_CULL_FACE);
}

FORCE_INLINE void xGraphicsReset2DMode()
{
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_DEPTH_TEST);
	//glDepthMask(GL_TRUE);

	//glEnable(GL_CULL_FACE);
	//glCullFace(GL_FRONT);

	gluLookAt(0.0, 0.0, 10.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

	glClear(GL_DEPTH_BUFFER_BIT);
}

#endif